PREFIX=arm-none-eabi-
CC = $(PREFIX)gcc
CXX = $(PREFIX)g++
LD = $(PREFIX)ld 
AR	= $(PREFIX)gcc-ar
RANLIB	= $(PREFIX)gcc-ranlib
OBJCOPY = $(PREFIX)objcopy

FLTO = -flto -fno-fat-lto-objects -flto-partition=none -fwhole-program -fvisibility=internal -flinker-output=nolto-rel 
LDFLAGS =  $(FLTO) -DNDEBUG -ggdb3  -Os -mcpu=cortex-m7 -mthumb -mfpu=fpv5-sp-d16 -mfloat-abi=hard -fno-common -fdata-sections -ffunction-sections -fno-exceptions -fno-strict-aliasing -fno-common -fno-threadsafe-statics -mthumb-interwork -Wl,--gc-sections -Wl,--entry=entrypoint --specs=nano.specs -nostartfiles -Wl,-Ur -lc -lstdc++ -lm  -lnosys
CXXFLAGS = -Os -I. -Igmpfr/include $(FLTO) # -I../../libtommath-0.39 
CXXFLAGS += -fno-threadsafe-statics -mthumb -march=armv7e-m -mfloat-abi=hard  -mcpu=cortex-m7 -mfpu=fpv5-sp-d16  -fdata-sections -ffunction-sections -fno-strict-aliasing -fno-exceptions # -fpermissive
#CXXFLAGS += -DNUMWORKS -DNSPIRE_NEWLIB -DHAVE_CONFIG_H -DIN_GIAC -DTIMEOUT -DNO_PHYSICAL_CONSTANTS -DNO_STDEXCEPT  -DGIAC_BINARY_ARCHIVE -DNO_UNARY_FUNCTION_COMPOSE -DNO_TEMPLATE_MULTGCD  -DNO_RTTI  -DGIAC_GENERIC_CONSTANTS # -DGIAC_NO_OPTIMIZATIONS -DSTATIC_BUILTIN_LEXER_FUNCTIONS
CFLAGS=-Os -mcpu=cortex-m7 -mthumb -mfpu=fpv5-sp-d16 -mfloat-abi=hard -fno-common -fdata-sections -ffunction-sections -fno-exceptions

OBJS = sym2poly.o gausspol.o threaded.o maple.o ti89.o mathml.o moyal.o misc.o permu.o desolve.o input_parser.o symbolic.o index.o modpoly.o modfactor.o ezgcd.o derive.o solve.o intg.o intgab.o risch.o lin.o series.o subst.o vecteur.o sparse.o csturm.o tex.o global.o ifactor.o alg_ext.o gauss.o isom.o help.o plot.o plot3d.o rpn.o prog.o pari.o cocoa.o unary.o usual.o identificateur.o gen.o input_lexer.o tinymt32.o first.o quater.o kdisplay.o kadd.o k_csdk.o startup.o api.o main.o qrcodegen.o 

#all: khi110ab.tar khi120ab.tar # run make cleanup and recompile with -DNUMWORKS_SLOTAB
#CXXFLAGS += -DDEVICE -DNUMWORKS -DNUMWORKS_SLOTAB -DHAVE_CONFIG_H -DIN_GIAC -DTIMEOUT -DNO_STDEXCEPT  -DGIAC_BINARY_ARCHIVE -DNO_UNARY_FUNCTION_COMPOSE -DNO_TEMPLATE_MULTGCD  -DGIAC_NO_OPTIMIZATIONS -DSTATIC_BUILTIN_LEXER_FUNCTIONS -DTIMEOUT -DMICROPY_LIB 

all: khi110b.tar khi120b.tar # run make cleanup recompile with -DNUMWORKS_SLOTB
CXXFLAGS += -DDEVICE -DNUMWORKS -DNUMWORKS_SLOTB -DHAVE_CONFIG_H -DIN_GIAC -DTIMEOUT -DNO_STDEXCEPT  -DGIAC_BINARY_ARCHIVE -DNO_UNARY_FUNCTION_COMPOSE -DNO_TEMPLATE_MULTGCD  -DGIAC_NO_OPTIMIZATIONS -DSTATIC_BUILTIN_LEXER_FUNCTIONS -DTIMEOUT -DMICROPY_LIB

cleanup:
	rm kdisplay.o help.o global.o kadd.o

.cc.o:
	$(CXX) $(CXXFLAGS) -c $<

.c.o:
	$(CC) $(CFLAGS) -c $<

khicas.elf: $(OBJS)
	$(CC) $(OBJS) gmpfr/lib/libmpfi.a gmpfr/lib/libmpfr.a gmpfr/lib/libgmp.a gmpfr/lib/libmicropy.a -o $@ $(LDFLAGS) # -Wl,-Map=khicas.map

khi120ab_linked.elf: khicas.elf n0120ab.ld
	$(LD) khicas.elf -o khi120ab_linked.elf -Tn0120ab.ld 
	objdump  -C -t khi120ab_linked.elf | sort > dump120ab_t

khi110ab_linked.elf: khicas.elf n0110ab.ld
	$(LD) khicas.elf -o khi110ab_linked.elf -Tn0110ab.ld 
	objdump  -C -t khi110ab_linked.elf | sort > dump110ab_t

OBJFLAGS = -O binary -R .bss -R .gint_bss -R .debug_info -R .debug_abbrev -R .debug_loc -R .debug_aranges -R .debug_ranges -R .debug_line -R .debug_str -R.comment -R  debug_frame
khi120ab: khi120ab_linked.elf Makefile
	$(OBJCOPY) $(OBJFLAGS) -R .rodata khi120ab_linked.elf khi120ab
	$(OBJCOPY) $(OBJFLAGS) -j .rodata khi120ab_linked.elf khi120a

khi120ab.tar: khi120ab
	cp -f khi120ab scripts && cd scripts && tar cvf ../khi120ab.tar khi120ab *.icon *.xw *.py

khi110ab: khi110ab_linked.elf Makefile
	$(OBJCOPY) $(OBJFLAGS) -R .rodata  khi110ab_linked.elf khi110ab
	$(OBJCOPY) $(OBJFLAGS) -j .rodata  khi110ab_linked.elf khi110a

khi110ab.tar: khi110ab
	cp -f khi110ab scripts && cd scripts && tar cvf ../khi110ab.tar khi110ab *.icon *.xw *.py

khi120b_linked.elf: khicas.elf n0120b.ld
	$(LD) khicas.elf -o khi120b_linked.elf -Tn0120b.ld 
	objdump  -C -t khi120b_linked.elf | sort > dump120b_t

khi110b_linked.elf: khicas.elf n0110b.ld
	$(LD) khicas.elf -o khi110b_linked.elf -Tn0110b.ld 
	objdump  -C -t khi110b_linked.elf | sort > dump110b_t

khi120b: khi120b_linked.elf Makefile
	$(OBJCOPY) $(OBJFLAGS) khi120b_linked.elf khi120b

khi120b.tar: khi120b
	cp -f khi120b scripts && cd scripts && tar cvf ../khi120b.tar khi120b 

khi110b: khi110b_linked.elf Makefile
	$(OBJCOPY) $(OBJFLAGS) khi110b_linked.elf khi110b

khi110b.tar: khi110b
	cp -f khi110b scripts && cd scripts && tar cvf ../khi110b.tar khi110b 

clean:
	rm -f *.o *.elf khi1?0* dump*_t 

libgiac.a: $(OBJS) 
	$(PREFIX)ar rcs libgiac.a $(OBJS) 
	$(PREFIX)ranlib libgiac.a
	$(PREFIX)objcopy --redefine-sym mp_init=mp_tm_init $@
